
 DROP PROCEDURE IF EXISTS tw_check_table;
 DELIMITER $$

 /**
 * TW_CHECK_TABLE  v0.1 02.06.2011
 * Prfen auf Primary Key und reservierte Worte 
 * siehe weiterhin source mysql.cc commands[] Array
 */
 CREATE PROCEDURE tw_check_table (IN p_sDatabase VARCHAR(64), 
                                  IN p_sTable VARCHAR(64) )
  SQL SECURITY INVOKER
  NOT DETERMINISTIC
  MODIFIES SQL DATA
  CONTAINS SQL
  COMMENT 'SQL-Backstube 02.06.2011 (www.twiedmann.de)'                                  
  
 BEGIN
 
  /* Variablen deklarieren */
  DECLARE have_primary_key INT DEFAULT 0;
  DECLARE have_help_keyword INT DEFAULT 0;
  DECLARE query_is_valid INT DEFAULT 1;
  
  /* Fetch Into Variablen */
  DECLARE sColumn_name VARCHAR(64) DEFAULT NULL;
  DECLARE sReserved_word VARCHAR(64) DEFAULT NULL;
  DECLARE sColumn_key VARCHAR(3) DEFAULT NULL;
  DECLARE sResult VARCHAR(2000) DEFAULT '';
  DECLARE nFetch_eof INT DEFAULT NULL;
  DECLARE nReserved_word INT DEFAULT 0;

  /* temporary result table */
  DECLARE sTempTableName VARCHAR(100) DEFAULT NULL;

  
  /* CURSOR - check MySQL columns und reserved words */
  DECLARE curCheckTable CURSOR FOR 
  SELECT UPPER(c.column_name) AS column_name, 
         UPPER(kw.name) AS reserved_word, 
         UPPER(c.column_key) AS column_key
    FROM information_schema.columns c
    LEFT JOIN mysql.help_keyword kw
      ON kw.name = c.column_name
   WHERE c.table_schema = p_sDatabase
     AND c.table_name = p_sTable; 

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET nFetch_eof = 1; 

  /* Build table name */
  SET sTempTableName = CONCAT('tw_temp_',MD5(NOW()));
  
  /* Create temporary table*/
  SET @sCommand = CONCAT('CREATE TEMPORARY TABLE ', sTempTableName ,'( ');
  SET @sCommand = CONCAT(@sCommand, ' id INT AUTO_INCREMENT PRIMARY KEY NOT NULL, ');
  SET @sCommand = CONCAT(@sCommand, ' msg VARCHAR(250) NOT NULL ');
  SET @sCommand = CONCAT(@sCommand, ' ); ');
  
  PREPARE stmt FROM @sCommand;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
  SET @sCommand = NULL;
 
  /* Parameter validieren */
  IF p_sDatabase IS NULL THEN
   SET query_is_valid = 0;
   SET @sCommand = CONCAT('INSERT INTO ',sTempTableName, '(msg) VALUES (', '''1. parameter IS NULL '' )' );
   PREPARE stmt FROM @sCommand;
   EXECUTE stmt;
   DEALLOCATE PREPARE stmt;
   SET @sCommand = NULL;
  END IF;
  
  IF p_sTable IS NULL THEN
   SET query_is_valid = 0;
   SET @sCommand = CONCAT('INSERT INTO ',sTempTableName, '(msg) VALUES (', '''2. parameter IS NULL '' )' );
   PREPARE stmt FROM @sCommand;
   EXECUTE stmt;
   DEALLOCATE PREPARE stmt;
   SET @sCommand = NULL;   
  END IF;  
  
  /* Existiert die Tabelle help_keyword ? */
  SELECT COUNT(*) 
    INTO have_help_keyword
    FROM information_schema.tables
   WHERE table_schema = 'mysql'
     AND table_name = 'help_keyword';
     
  IF have_help_keyword = 0 THEN
   SET query_is_valid = 0;
   SET @sCommand = CONCAT('INSERT INTO ',sTempTableName, '(msg) VALUES (', '''mysql.help_keyword not found'' )' );
   PREPARE stmt FROM @sCommand;
   EXECUTE stmt;
   DEALLOCATE PREPARE stmt;
   SET @sCommand = NULL;    
  END IF;
  
  /* Start der Prfung */
  IF query_is_valid = 1 THEN
   
   SET sResult = CONCAT('Check-Table: ', p_sDatabase, '.', p_sTable, '   Reserved-/Keyword(s): ');
   
   SET @sCommand = CONCAT('INSERT INTO ',sTempTableName, ' (msg) VALUES (''', sResult, ''' ) ' );
   PREPARE stmt FROM @sCommand;
   EXECUTE stmt;
   DEALLOCATE PREPARE stmt;
   SET @sCommand = NULL;      
   
   SET nFetch_eof = 0;
   
   OPEN curCheckTable; 
   allCheckTable: LOOP 
    FETCH curCheckTable INTO sColumn_name, sReserved_word, sColumn_key; 
    IF nFetch_eof THEN 
     LEAVE allCheckTable; 
    END IF;
    
    IF sReserved_word IS NOT NULL THEN
    
     SET sResult = CONCAT('Reserved Word: ', sReserved_word);    
     SET @sCommand = CONCAT('INSERT INTO ',sTempTableName, ' (msg) VALUES (''', sResult, ''' ) ' );
     PREPARE stmt FROM @sCommand;
     EXECUTE stmt;
     DEALLOCATE PREPARE stmt;
     SET @sCommand = NULL;      
     
     SET nReserved_word = nReserved_word +1;
    END IF;
    
    IF sColumn_key = 'PRI' THEN
     SET have_primary_key = 1;
    END IF;    

   END LOOP allCheckTable;
   
   /* CLOSE Cursor */
   CLOSE curCheckTable; 

   /* abschliessende Auswertungen, Formatierungen und Primary Key */
   IF nReserved_word = 0 THEN
   
    SET sResult = CONCAT('Reserved Word: ok');    
    SET @sCommand = CONCAT('INSERT INTO ',sTempTableName, ' (msg) VALUES (''', sResult, ''' ) ' );
    PREPARE stmt FROM @sCommand;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
    SET @sCommand = NULL;       
   
   END IF;

   IF have_primary_key = 0 THEN
    SET sResult = 'Primary Key: NOT exists ';
   ELSE 
    SET sResult = 'Primary Key: exists ';    
   END IF;
   
   SET @sCommand = CONCAT('INSERT INTO ',sTempTableName, ' (msg) VALUES (''', sResult, ''' ) ' );
   PREPARE stmt FROM @sCommand;
   EXECUTE stmt;
   DEALLOCATE PREPARE stmt;
   SET @sCommand = NULL;     
   
  END IF;

  /*  Ausgabe  */
  SET @sCommand = CONCAT('SELECT * FROM ',sTempTableName, ' ORDER BY id' );
  PREPARE stmt FROM @sCommand;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
  SET @sCommand = NULL;  


  /* DROP */
  SET @sCommand = CONCAT('DROP TEMPORARY TABLE ',sTempTableName );
  PREPARE stmt FROM @sCommand;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
  SET @sCommand = NULL;    
  
 END$$
 
 DELIMITER ;
 